<?php

/**
* class Classe
*
* @version 1.0.3
* @author Baptiste ROUSSEL
* @license MPL 2.0
* This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0. 
* If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
*/

class Classe implements Iterator{
	private static $classes = array();

	public $id = 0;
	public $name = '';
	public $description = '';
	public $abilities = array();

	public function __construct( $id = 0 )
	{
		if ( !is_numeric( $id ) )
			throw new Exception( 'Classe\'s ID must be a number.' );
		if( $id == 0 )return;
		if ( $this->isClasseLoaded( $id ) === false ) {
			// Load the Classe from the database
			$this->id = $id;
			$db = Database::getDatabase();
			$res = $db->prepare( 'SELECT `name`, `description` FROM `' . TBL_CLA . '` WHERE `idclasse` = :id' );
			$res->bindValue( ':id', $id );
			$res->execute();
			if ( $res->rowCount() == 1 ) {
				$f = $res->fetch();
				$this->name = $f['name'];
				$this->description = $f['description'];
				$res->closeCursor();
			} else
				throw new Exception( 'Unknow Classe ' . $id . '.' );
			// Load the Classe's abilities
			$res = $db->prepare( 'SELECT `idability` AS `id`, `dice`, `bonusvalue` FROM `' . TBL_CLA_AB . '` WHERE `idclasse` = :id ORDER BY `idability`' );
			$res->bindValue( ':id', $id );
			$res->execute();
			if ( $res->rowCount() > 0 ) {
				foreach( $res as $ab ) {
					if ( Ability::isAbilityLoaded( $ab['id'] ) === false )
						new Ability( $ab['id'] );
						$this->abilities[$ab['id']] = array(
							'dice' => new Dice( $ab['dice'] ),
							'bonusValue' => $ab['bonusvalue']
						);
				}
			}

			self::$classes[$id] = $this;
		} else {
			// Copy of the Classe already loaded
			$classe = self::$classes[$id];
			$this->id = $id;
			$this->name = $classe->name;
			$this->description = $classe->description;
		}
	}

	/*
	   * @param (int) Classe's ID
	   * @return (Classe)
	*/
	public static function getClasse( $id )
	{
		if ( !is_numeric( $id ) )
			throw new Exception( 'Classe\'s ID must be a number.' );
		if ( self::isClasseLoaded( $id ) )
			return self::$classes[$id];
		else {
			return new Classe( $id );
		}
	}

	/*
	* Return the first classe loaded
	* @return (Skill) or null if no classe loaded
	*/
	public static function getFirst(){
		$key = array_keys(self::$classes);
		if( sizeof($key) == 0 )
			return null;
		return self::$classes[$key[0]];
	}

	/*
	   * @param (int) Classe's ID
	   * @return (bool)
	*/
	private static function isClasseLoaded( $id )
	{
		if ( !is_numeric( $id ) )
			throw new Exception( 'Classe\'s ID must be a number.' );
		return !empty( self::$classes[$id] );
	}

	/*
	   * Load a classe from the database
	   * @param (int) Classe's ID
	   * @return (bool)
	*/
	private static function loadClasse( $id )
	{
		if ( !is_numeric( $id ) )
			throw new Exception( 'Classe\'s ID must be a number.' );
		if ( empty( self::$classes[$id] ) )
			new Classe( $id );
		return true;
	}

	public static function loadAll(){
		$db = Database::getDatabase();
		$res = $db->query('SELECT `idclasse` FROM `' . TBL_CLA . '`');
		if( $res->rowCount() > 0 ){
			foreach($res as $r)
				self::loadClasse($r['idclasse']);
		}
	}

	/*
	   * Getter
	   * @param (String) Variable name
	   * @return (mixed) Value
	*/
	public function __get( $name )
	{
		if ( isSet( $this->$name ) && $name != 'classes' && $name != 'abilities' )
			return $this->$name;
		return null;
	}

	public function getAbility( $id ){
		if ( !is_numeric( $id ) )
			throw new Exception( 'Ability\'s ID must be a number.' );
		if( isSet($this->abilities[$id]) )
			return $this->abilities[$id];
		else
			return false;
	}

	public function save(){
		$db = Database::getDatabase();
		$db->beginTransaction();
		// Look if it is an update or a creation
		$update = ($this->id != 0) ? true : false;
		// table : classe
		if( $update === true ){
			$res = $db->prepare('UPDATE `' . TBL_CLA . '` SET `name` = :name, `description` = :description WHERE `idclasse` = :id');
			$res->bindValue(':name', $this->name, PDO::PARAM_STR);
			$res->bindValue(':description', $this->description, PDO::PARAM_STR);
			$res->bindValue(':id', $this->id, PDO::PARAM_INT);
			$res->execute();
		}else{
			$res = $db->prepare('INSERT into `' . TBL_CLA . '`(`name`,`description`) VALUES (:name, :description)');
			$res->bindValue(':name', $this->name, PDO::PARAM_STR);
			$res->bindValue(':description', $this->description, PDO::PARAM_STR);
			$res->execute();
			$this->id = $db->lastInsertId();
		}
		// table : classe_ability
		$db->query('DELETE FROM `' . TBL_CLA_AB . '` WHERE `idclasse` = ' . $db->quote($this->id));
		$res = $db->prepare('INSERT into `' . TBL_CLA_AB . '`(`dice`,`bonusvalue`,`idclasse`,`idability`) VALUES(:dice, :bonusvalue, :idclasse, :idability)');
		if( !empty($this->abilities) )
			foreach($this->abilities as $id => $ab){
				$res->bindValue(':dice',$ab['dice']->getDice(),PDO::PARAM_STR);
				if( !isSet($ab['bonusValue']) )
					$ab['bonusValue'] = 0;
				$res->bindValue(':bonusvalue',$ab['bonusValue'],PDO::PARAM_INT);
				$res->bindValue(':idclasse',$this->id,PDO::PARAM_INT);
				$res->bindValue(':idability',$id,PDO::PARAM_INT);
				$res->execute();
			}
		$db->commit();
		return true;
	}

	public function delete(){
		$db = Database::getDatabase();
		$db->beginTransaction();
		$res = $db->query('SELECT count(`iddata`) AS `nb` FROM `' . TBL_D . '` WHERE `idclasse` != ' . $db->quote($this->id));
		if( $type = $res->fetch() ){
			if( $type['nb'] > 0 ){ // we don't delete anything if it's in use
				return false;
			}
		}
		$db->query('DELETE FROM `' . TBL_CLA_AB . '` WHERE `idclasse` = ' . $db->quote($this->id));
		$db->query('DELETE FROM `' . TBL_CLA . '` WHERE `idclasse` = ' . $db->quote($this->id));
		$db->commit();
		return true;
	}

	public function rewind() {
		reset(self::$classes);
	}

	public function current() {
		return current(self::$classes);
	}

	public function key() {
		return key(self::$classes);
	}

	public function next() {
		next(self::$classes);
	}

	public function valid() {
		return (current(self::$classes) != false);
	}
}

?>