<?php

/**
* class Race
*
* @version 1.0.3
* @author Baptiste ROUSSEL
* @license MPL 2.0
* This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0. 
* If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
*/

class Type implements Iterator{
	private static $types = array();

	public $id = 0;
	public $name = '';
	public $description = '';
	public $abilities = array();

	public function __construct( $id = 0 )
	{
		if ( !is_numeric( $id ) )
			throw new Exception( 'Type\'s ID must be a number.' );
		if( $id === 0 )return;
		if ( $this->isTypeLoaded( $id ) === false ) {
			// Load the type from the database
			$this->id = $id;
			$db = Database::getDatabase();
			$res = $db->prepare( 'SELECT `name`, `description` FROM `' . TBL_T . '` WHERE `idtype` = :id' );
			$res->bindValue( ':id', $id );
			$res->execute();
			if ( $res->rowCount() == 1 ) {
				$f = $res->fetch();
				$this->name = $f['name'];
				$this->description = $f['description'];
				$res->closeCursor();
			} else
				throw new Exception( 'Unknow Type ' . $id . '.' );
			// Load the type's abilities
			$res = $db->prepare( 'SELECT `idability` AS `id`, `dice`, `bonusvalue` FROM `' . TBL_T_AB . '` WHERE `idtype` = :id ORDER BY `idability`' );
			$res->bindValue( ':id', $id );
			$res->execute();
			if ( $res->rowCount() > 0 ) {
				foreach( $res as $ab ) {
					if ( Ability::isAbilityLoaded( $ab['id'] ) === false )
						new Ability( $ab['id'] );
						$this->abilities[$ab['id']] = array(
							'dice' => new Dice( $ab['dice'] ),
							'bonusValue' => $ab['bonusvalue']
						);
				}
			}

			self::$types[$id] = $this;
		} else {
			// Copy of the Type already loaded
			$type = self::$types[$id];
			$this->id = $id;
			$this->name = $type->name;
			$this->description = $race->description;
		}
	}

	/*
	* @param (int) Type's ID
	* @return (Type)
	*/
	public static function getType( $id )
	{
		if ( !is_numeric( $id ) )
			throw new Exception( 'Type\'s ID must be a number.' );
		if ( self::isTypeLoaded( $id ) )
			return self::$types[$id];
		else {
			return new Type( $id );
		}
	}

	/*
	* Return the first Type loaded
	* @return (Skill) or null if no type loaded
	*/
	public static function getFirst(){
		$key = array_keys(self::$types);
		if( sizeof($key) == 0 )
			return null;
		return self::$types[$key[0]];
	}

	/*
	* @param (int) Type's ID
	* @return (bool)
	*/
	private static function isTypeLoaded( $id )
	{
		if ( !is_numeric( $id ) )
			throw new Exception( 'Type\'s ID must be a number.' );
		return !empty( self::$types[$id] );
	}

	/*
	* Load a type from the database
	* @param (int) Type's ID
	* @return (bool)
	*/
	private static function loadType( $id )
	{
		if ( !is_numeric( $id ) )
			throw new Exception( 'Type\'s ID must be a number.' );
		if ( empty( self::$types[$id] ) )
			new Type( $id );
		return true;
	}

	public static function loadAll(){
		$db = Database::getDatabase();
		$res = $db->query('SELECT `idtype` FROM `' . TBL_T . '`');
		if( $res->rowCount() > 0 ){
			foreach($res as $r)
				self::loadType($r['idtype']);
		}
	}

	/*
	* Getter
	* @param (String) Variable name
	* @return (mixed) Value
	*/
	public function __get( $name )
	{
		if ( isSet( $this->$name ) && $name != 'types' && $name != 'abilities' )
			return $this->$name;
		return null;
	}

	public function getAbility( $id ){
		if ( !is_numeric( $id ) )
			throw new Exception( 'Ability\'s ID must be a number.' );
		if( isSet($this->abilities[$id]) )
			return $this->abilities[$id];
		else
			return false;
	}

	public function save(){
		$db = Database::getDatabase();
		$db->beginTransaction();
		// Look if it is an update or a creation
		$update = ($this->id != 0) ? true : false;
		// table : type
		if( $update === true ){
			$res = $db->prepare('UPDATE `' . TBL_T . '` SET `name` = :name, `description` = :description WHERE `idtype` = :id');
			$res->bindValue(':name', $this->name, PDO::PARAM_STR);
			$res->bindValue(':description', $this->description, PDO::PARAM_STR);
			$res->bindValue(':id', $this->id, PDO::PARAM_INT);
			$res->execute();
		}else{
			$res = $db->prepare('INSERT into `' . TBL_T . '`(`name`,`description`) VALUES (:name, :description)');
			$res->bindValue(':name', $this->name, PDO::PARAM_STR);
			$res->bindValue(':description', $this->description, PDO::PARAM_STR);
			$res->execute();
			$this->id = $db->lastInsertId();
		}
		// table : type_ability
		$db->query('DELETE FROM `' . TBL_T_AB . '` WHERE `idtype` = ' . $db->quote($this->id));
		$res = $db->prepare('INSERT into `' . TBL_T_AB . '`(`dice`,`bonusvalue`,`idtype`,`idability`) VALUES(:dice, :bonusvalue, :idtype, :idability)');
		if( !empty($this->abilities) )
		foreach($this->abilities as $id => $ab){
			$res->bindValue(':dice',$ab['dice']->getDice(),PDO::PARAM_STR);
			if( !isSet($ab['bonusValue']) )
				$ab['bonusValue'] = 0;
			$res->bindValue(':bonusvalue',$ab['bonusValue'],PDO::PARAM_INT);
			$res->bindValue(':idtype',$this->id,PDO::PARAM_INT);
			$res->bindValue(':idability',$id,PDO::PARAM_INT);
			$res->execute();
		}
		$db->commit();
		return true;
	}

	public function delete(){
		$db = Database::getDatabase();
		$db->beginTransaction();
		$res = $db->query('SELECT count(`iddata`) AS `nb` FROM `' . TBL_D . '` WHERE `idtype` != ' . $db->quote($this->id));
		if( $type = $res->fetch() ){
			if( $type['nb'] > 0 ){ // we don't delete anything if it's in use
				return false;
			}
		}
		$db->query('DELETE FROM `' . TBL_T_AB . '` WHERE `idtype` = ' . $db->quote($this->id));
		$db->query('DELETE FROM `' . TBL_T . '` WHERE `idtype` = ' . $db->quote($this->id));
		$db->commit();
		return true;
	}

	public function rewind() {
		reset(self::$types);
	}

	public function current() {
		return current(self::$types);
	}

	public function key() {
		return key(self::$types);
	}

	public function next() {
		next(self::$types);
	}

	public function valid() {
		return (current(self::$types) != false);
	}
}

?>